@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 
@@ -78,8 +78,6 @@ struct _GstDtsDecClass {
 
 GType gst_dtsdec_get_type(void);
 
-GST_ELEMENT_REGISTER_DECLARE (dtsdec);
-
 G_END_DECLS
 
 #endif /* __GST_DTSDEC_H__ */
