@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
@@ -32,12 +32,16 @@
  * <refsect2>
  * <title>Example pipeline</title>
  * |[
- * gst-launch-1.0 filesrc location=song.mid ! wildmidi ! alsasink
+ * gst-launch filesrc location=song.mid ! wildmidi ! alsasink
  * ]| This example pipeline will parse the midi and render to raw audio which is
  * played via alsa.
  * </refsect2>
  */
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -80,23 +84,18 @@ enum
 
 static void gst_wildmidi_finalize (GObject * object);
 
-static gboolean gst_wildmidi_sink_event (GstPad * pad, GstObject * parent,
-    GstEvent * event);
-static gboolean gst_wildmidi_src_event (GstPad * pad, GstObject * parent,
-    GstEvent * event);
+static gboolean gst_wildmidi_sink_event (GstPad * pad, GstEvent * event);
+static gboolean gst_wildmidi_src_event (GstPad * pad, GstEvent * event);
 
 static GstStateChangeReturn gst_wildmidi_change_state (GstElement * element,
     GstStateChange transition);
-static gboolean gst_wildmidi_activate (GstPad * pad, GstObject * parent);
-static gboolean gst_wildmidi_activatemode (GstPad * pad, GstObject * parent,
-    GstPadMode mode, gboolean active);
+static gboolean gst_wildmidi_activate (GstPad * pad);
+static gboolean gst_wildmidi_activatepull (GstPad * pad, gboolean active);
 
 static void gst_wildmidi_loop (GstPad * sinkpad);
-static GstFlowReturn gst_wildmidi_chain (GstPad * sinkpad, GstObject * parent,
-    GstBuffer * buffer);
+static GstFlowReturn gst_wildmidi_chain (GstPad * sinkpad, GstBuffer * buffer);
 
-static gboolean gst_wildmidi_src_query (GstPad * pad, GstObject * parent,
-    GstQuery * query);
+static gboolean gst_wildmidi_src_query (GstPad * pad, GstQuery * query);
 
 static void gst_wildmidi_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -112,13 +111,27 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("audio/x-raw, "
-        "format = (string) S16LE, "
+    GST_STATIC_CAPS ("audio/x-raw-int, "
         "rate = (int) 44100, "
-        "channels = (int) 2, " "layout = (string) interleaved"));
+        "channels = (int) 2, "
+        "endianness = (int) LITTLE_ENDIAN, "
+        "width = (int) 16, " "depth = (int) 16, " "signed = (boolean) true"));
 
-#define parent_class gst_wildmidi_parent_class
-G_DEFINE_TYPE (GstWildmidi, gst_wildmidi, GST_TYPE_ELEMENT);
+GST_BOILERPLATE (GstWildmidi, gst_wildmidi, GstElement, GST_TYPE_ELEMENT);
+
+static void
+gst_wildmidi_base_init (gpointer gclass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_set_static_metadata (element_class, "WildMidi",
+      "Codec/Decoder/Audio",
+      "Midi Synthesizer Element", "Wouter Paesen <wouter@blue-gate.be>");
+}
 
 static gboolean
 wildmidi_open_config (void)
@@ -239,14 +252,6 @@ gst_wildmidi_class_init (GstWildmidiClass * klass)
           "High Quality", DEFAULT_HIGH_QUALITY,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_factory));
-  gst_element_class_set_static_metadata (gstelement_class, "WildMidi",
-      "Codec/Decoder/Audio",
-      "Midi Synthesizer Element", "Wouter Paesen <wouter@blue-gate.be>");
-
   gstelement_class->change_state = gst_wildmidi_change_state;
 }
 
@@ -256,18 +261,24 @@ gst_wildmidi_class_init (GstWildmidiClass * klass)
  * initialize structure
  */
 static void
-gst_wildmidi_init (GstWildmidi * filter)
+gst_wildmidi_init (GstWildmidi * filter, GstWildmidiClass * g_class)
 {
-  filter->sinkpad = gst_pad_new_from_static_template (&sink_factory, "sink");
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (filter);
 
-  gst_pad_set_activatemode_function (filter->sinkpad,
-      gst_wildmidi_activatemode);
+  filter->sinkpad =
+      gst_pad_new_from_template (gst_element_class_get_pad_template (klass,
+          "sink"), "sink");
+
+  gst_pad_set_activatepull_function (filter->sinkpad,
+      gst_wildmidi_activatepull);
   gst_pad_set_activate_function (filter->sinkpad, gst_wildmidi_activate);
   gst_pad_set_event_function (filter->sinkpad, gst_wildmidi_sink_event);
   gst_pad_set_chain_function (filter->sinkpad, gst_wildmidi_chain);
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
 
-  filter->srcpad = gst_pad_new_from_static_template (&src_factory, "src");
+  filter->srcpad =
+      gst_pad_new_from_template (gst_element_class_get_pad_template (klass,
+          "src"), "src");
 
   gst_pad_set_query_function (filter->srcpad, gst_wildmidi_src_query);
   gst_pad_set_event_function (filter->srcpad, gst_wildmidi_src_event);
@@ -346,15 +357,17 @@ done:
 }
 
 static gboolean
-gst_wildmidi_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
+gst_wildmidi_src_query (GstPad * pad, GstQuery * query)
 {
   gboolean res = TRUE;
-  GstWildmidi *wildmidi = GST_WILDMIDI (parent);
+  GstWildmidi *wildmidi = GST_WILDMIDI (gst_pad_get_parent (pad));
   GstFormat src_format, dst_format;
   gint64 src_value, dst_value;
 
-  if (!wildmidi->song)
+  if (!wildmidi->song) {
+    gst_object_unref (wildmidi);
     return FALSE;
+  }
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_DURATION:
@@ -364,7 +377,7 @@ gst_wildmidi_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       break;
     case GST_QUERY_POSITION:
       gst_query_set_position (query, GST_FORMAT_TIME,
-          gst_util_uint64_scale_int (wildmidi->o_segment->position, GST_SECOND,
+          gst_util_uint64_scale_int (wildmidi->o_segment->last_stop, GST_SECOND,
               WILDMIDI_RATE));
       break;
     case GST_QUERY_CONVERT:
@@ -383,48 +396,34 @@ gst_wildmidi_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       gst_query_set_formats (query, 3,
           GST_FORMAT_TIME, GST_FORMAT_BYTES, GST_FORMAT_DEFAULT);
       break;
-    case GST_QUERY_SEGMENT:{
-      GstFormat format;
-      gint64 start, stop;
-
-      format = wildmidi->o_segment->format;
-
-      start =
-          gst_segment_to_stream_time (wildmidi->o_segment, format,
-          wildmidi->o_segment->start);
-      if ((stop = wildmidi->o_segment->stop) == -1)
-        stop = wildmidi->o_segment->duration;
-      else
-        stop = gst_segment_to_stream_time (wildmidi->o_segment, format, stop);
-
-      gst_query_set_segment (query, wildmidi->o_segment->rate, format, start,
-          stop);
-      res = TRUE;
+    case GST_QUERY_SEGMENT:
+      gst_query_set_segment (query, wildmidi->o_segment->rate,
+          wildmidi->o_segment->format, wildmidi->o_segment->start,
+          wildmidi->o_segment->stop);
       break;
-    }
     case GST_QUERY_SEEKING:
       gst_query_set_seeking (query, wildmidi->o_segment->format,
           TRUE, 0, wildmidi->o_len);
       break;
     default:
-      res = gst_pad_query_default (pad, parent, query);
+      res = FALSE;
       break;
   }
 
+  gst_object_unref (wildmidi);
   return res;
 }
 
 static GstEvent *
 gst_wildmidi_get_new_segment_event (GstWildmidi * wildmidi, GstFormat format)
 {
-  gint64 start = 0, stop = -1, time = 0;
-  GstSegment *segment, newseg;
+  gint64 start, stop, time;
+  GstSegment *segment;
   GstEvent *event;
   GstFormat src_format;
 
   segment = wildmidi->o_segment;
   src_format = segment->format;
-  newseg = *segment;
 
   /* convert the segment values to the target format */
   gst_wildmidi_src_convert (wildmidi, src_format, segment->start, &format,
@@ -434,12 +433,8 @@ gst_wildmidi_get_new_segment_event (GstWildmidi * wildmidi, GstFormat format)
   gst_wildmidi_src_convert (wildmidi, src_format, segment->time, &format,
       &time);
 
-  newseg.format = format;
-  newseg.start = start;
-  newseg.stop = stop;
-  newseg.time = time;
-
-  event = gst_event_new_segment (&newseg);
+  event = gst_event_new_new_segment_full (FALSE,
+      segment->rate, segment->applied_rate, format, start, stop, time);
 
   return event;
 }
@@ -500,15 +495,15 @@ gst_wildmidi_do_seek (GstWildmidi * wildmidi, GstEvent * event)
   GST_PAD_STREAM_LOCK (wildmidi->sinkpad);
 
   if (flush) {
-    gst_pad_push_event (wildmidi->srcpad, gst_event_new_flush_stop (TRUE));
+    gst_pad_push_event (wildmidi->srcpad, gst_event_new_flush_stop ());
   }
 
   /* update the segment now */
-  gst_segment_do_seek (segment, rate, dst_format, flags,
+  gst_segment_set_seek (segment, rate, dst_format, flags,
       start_type, start, stop_type, stop, &update);
 
-  /* we need to seek to position in the segment now, sample will be updated */
-  sample = segment->position;
+  /* we need to seek to last_stop in the segment now, sample will be updated */
+  sample = segment->last_stop;
 
   GST_OBJECT_LOCK (wildmidi);
 #ifdef HAVE_WILDMIDI_0_2_2
@@ -523,7 +518,7 @@ gst_wildmidi_do_seek (GstWildmidi * wildmidi, GstEvent * event)
 
   GST_OBJECT_UNLOCK (wildmidi);
 
-  segment->start = segment->time = segment->position = sample;
+  segment->start = segment->time = segment->last_stop = sample;
 
   gst_pad_push_event (wildmidi->srcpad,
       gst_wildmidi_get_new_segment_event (wildmidi, GST_FORMAT_TIME));
@@ -539,10 +534,10 @@ gst_wildmidi_do_seek (GstWildmidi * wildmidi, GstEvent * event)
 }
 
 static gboolean
-gst_wildmidi_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
+gst_wildmidi_src_event (GstPad * pad, GstEvent * event)
 {
   gboolean res = FALSE;
-  GstWildmidi *wildmidi = GST_WILDMIDI (parent);
+  GstWildmidi *wildmidi = GST_WILDMIDI (gst_pad_get_parent (pad));
 
   GST_DEBUG_OBJECT (pad, "%s event received", GST_EVENT_TYPE_NAME (event));
 
@@ -553,73 +548,39 @@ gst_wildmidi_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
     default:
       break;
   }
-  gst_event_unref (event);
 
+  g_object_unref (wildmidi);
   return res;
 }
 
 
 static gboolean
-gst_wildmidi_activate (GstPad * sinkpad, GstObject * parent)
+gst_wildmidi_activate (GstPad * sinkpad)
 {
-  GstQuery *query;
-  gboolean pull_mode;
+  if (gst_pad_check_pull_range (sinkpad))
+    return gst_pad_activate_pull (sinkpad, TRUE);
 
-  query = gst_query_new_scheduling ();
-
-  if (!gst_pad_peer_query (sinkpad, query)) {
-    gst_query_unref (query);
-    goto activate_push;
-  }
-
-  pull_mode = gst_query_has_scheduling_mode_with_flags (query,
-      GST_PAD_MODE_PULL, GST_SCHEDULING_FLAG_SEEKABLE);
-  gst_query_unref (query);
-
-  if (!pull_mode)
-    goto activate_push;
-
-  GST_DEBUG_OBJECT (sinkpad, "activating pull");
-  return gst_pad_activate_mode (sinkpad, GST_PAD_MODE_PULL, TRUE);
-
-activate_push:
-  {
-    GST_DEBUG_OBJECT (sinkpad, "activating push");
-    return gst_pad_activate_mode (sinkpad, GST_PAD_MODE_PUSH, TRUE);
-  }
+  return gst_pad_activate_push (sinkpad, TRUE);
 }
 
 static gboolean
-gst_wildmidi_activatemode (GstPad * pad, GstObject * parent,
-    GstPadMode mode, gboolean active)
+gst_wildmidi_activatepull (GstPad * pad, gboolean active)
 {
-  gboolean res;
-
-  switch (mode) {
-    case GST_PAD_MODE_PUSH:
-      res = TRUE;
-      break;
-    case GST_PAD_MODE_PULL:
-      if (active) {
-        res = gst_pad_start_task (pad, (GstTaskFunction) gst_wildmidi_loop,
-            pad, NULL);
-      } else {
-        res = gst_pad_stop_task (pad);
-      }
-      break;
-    default:
-      res = FALSE;
-      break;
+  if (active) {
+    return gst_pad_start_task (pad, (GstTaskFunction) gst_wildmidi_loop, pad,
+        NULL);
+  } else {
+    return gst_pad_stop_task (pad);
   }
-  return res;
 }
 
 static GstBuffer *
 gst_wildmidi_clip_buffer (GstWildmidi * wildmidi, GstBuffer * buffer)
 {
-  guint64 start, stop;
-  guint64 new_start, new_stop;
+  gint64 start, stop;
+  gint64 new_start, new_stop;
   gint64 offset, length;
+  GstBuffer *out;
   guint64 bpf;
 
   /* clipping disabled for now */
@@ -642,64 +603,65 @@ gst_wildmidi_clip_buffer (GstWildmidi * wildmidi, GstBuffer * buffer)
   length = new_stop - new_start;
 
   bpf = wildmidi->bytes_per_frame;
-  buffer = gst_buffer_make_writable (buffer);
-  gst_buffer_resize (buffer, offset * bpf, length * bpf);
+  out = gst_buffer_create_sub (buffer, offset * bpf, length * bpf);
 
-  GST_BUFFER_OFFSET (buffer) = new_start;
-  GST_BUFFER_OFFSET_END (buffer) = new_stop;
-  GST_BUFFER_TIMESTAMP (buffer) =
+  GST_BUFFER_OFFSET (out) = new_start;
+  GST_BUFFER_OFFSET_END (out) = new_stop;
+  GST_BUFFER_TIMESTAMP (out) =
       gst_util_uint64_scale_int (new_start, GST_SECOND, WILDMIDI_RATE);
-  GST_BUFFER_DURATION (buffer) =
+  GST_BUFFER_DURATION (out) =
       gst_util_uint64_scale_int (new_stop, GST_SECOND, WILDMIDI_RATE) -
-      GST_BUFFER_TIMESTAMP (buffer);
+      GST_BUFFER_TIMESTAMP (out);
 
-  return buffer;
+  gst_buffer_unref (buffer);
+
+  return out;
 }
 
 /* generate audio data and advance internal timers */
 static GstBuffer *
 gst_wildmidi_get_buffer (GstWildmidi * wildmidi)
 {
-  size_t size;
+  size_t bytes_read;
   gint64 samples;
   GstBuffer *buffer;
   GstSegment *segment;
-  GstMapInfo info;
+  guint8 *data;
+  guint size;
   guint bpf;
 
   bpf = wildmidi->bytes_per_frame;
 
   buffer = gst_buffer_new_and_alloc (256 * bpf);
 
-  gst_buffer_map (buffer, &info, GST_MAP_READWRITE);
+  data = GST_BUFFER_DATA (buffer);
+  size = GST_BUFFER_SIZE (buffer);
 
   GST_OBJECT_LOCK (wildmidi);
-  size = WildMidi_GetOutput (wildmidi->song, (char *) info.data,
-      (unsigned long int) info.size);
+  bytes_read = WildMidi_GetOutput (wildmidi->song, (char *) data,
+      (unsigned long int) size);
   GST_OBJECT_UNLOCK (wildmidi);
 
-  gst_buffer_unmap (buffer, &info);
-
-  if (size == 0) {
+  if (bytes_read == 0) {
     gst_buffer_unref (buffer);
     return NULL;
   }
 
   /* adjust buffer size */
-  gst_buffer_resize (buffer, 0, size);
+  size = GST_BUFFER_SIZE (buffer) = bytes_read;
 
   segment = wildmidi->o_segment;
 
-  GST_BUFFER_OFFSET (buffer) = segment->position;
+  GST_BUFFER_OFFSET (buffer) = segment->last_stop;
   GST_BUFFER_TIMESTAMP (buffer) =
-      gst_util_uint64_scale_int (segment->position, GST_SECOND, WILDMIDI_RATE);
+      gst_util_uint64_scale_int (segment->last_stop, GST_SECOND, WILDMIDI_RATE);
 
   samples = size / bpf;
-  segment->position += samples;
+  segment->last_stop += samples;
 
-  GST_BUFFER_OFFSET_END (buffer) = segment->position;
+  GST_BUFFER_OFFSET_END (buffer) = segment->last_stop;
   GST_BUFFER_DURATION (buffer) =
-      gst_util_uint64_scale_int (segment->position, GST_SECOND,
+      gst_util_uint64_scale_int (segment->last_stop, GST_SECOND,
       WILDMIDI_RATE) - GST_BUFFER_TIMESTAMP (buffer);
 
   GST_DEBUG_OBJECT (wildmidi, "buffer ts: %" GST_TIME_FORMAT ", "
@@ -756,7 +718,8 @@ gst_wildmidi_parse_song (GstWildmidi * wildmidi)
   gst_caps_unref (outcaps);
 
   /* we keep an internal segment in samples */
-  gst_segment_init (wildmidi->o_segment, GST_FORMAT_DEFAULT);
+  gst_segment_set_newsegment (wildmidi->o_segment, FALSE, 1.0,
+      GST_FORMAT_DEFAULT, 0, GST_CLOCK_TIME_NONE, 0);
 
   gst_pad_push_event (wildmidi->srcpad,
       gst_wildmidi_get_new_segment_event (wildmidi, GST_FORMAT_TIME));
@@ -789,6 +752,7 @@ gst_wildmidi_do_play (GstWildmidi * wildmidi)
     wildmidi->discont = FALSE;
   }
 
+  gst_buffer_set_caps (out, GST_PAD_CAPS (wildmidi->srcpad));
   ret = gst_pad_push (wildmidi->srcpad, out);
 
   return ret;
@@ -797,15 +761,15 @@ gst_wildmidi_do_play (GstWildmidi * wildmidi)
 eos:
   {
     GST_LOG_OBJECT (wildmidi, "Song ended");
-    return GST_FLOW_EOS;
+    return GST_FLOW_UNEXPECTED;
   }
 }
 
 static gboolean
-gst_wildmidi_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
+gst_wildmidi_sink_event (GstPad * pad, GstEvent * event)
 {
-  gboolean res;
-  GstWildmidi *wildmidi = GST_WILDMIDI (parent);
+  gboolean res = FALSE;
+  GstWildmidi *wildmidi = GST_WILDMIDI (gst_pad_get_parent (pad));
 
   GST_DEBUG_OBJECT (pad, "%s event received", GST_EVENT_TYPE_NAME (event));
 
@@ -813,24 +777,26 @@ gst_wildmidi_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
     case GST_EVENT_EOS:
       wildmidi->state = GST_WILDMIDI_STATE_PARSE;
       /* now start the parsing task */
-      res = gst_pad_start_task (wildmidi->sinkpad,
+      gst_pad_start_task (wildmidi->sinkpad,
           (GstTaskFunction) gst_wildmidi_loop, wildmidi->sinkpad, NULL);
       /* don't forward the event */
       gst_event_unref (event);
       break;
     default:
-      res = gst_pad_event_default (pad, parent, event);
+      res = gst_pad_push_event (wildmidi->srcpad, event);
       break;
   }
+
+  gst_object_unref (wildmidi);
   return res;
 }
 
 static GstFlowReturn
-gst_wildmidi_chain (GstPad * sinkpad, GstObject * parent, GstBuffer * buffer)
+gst_wildmidi_chain (GstPad * sinkpad, GstBuffer * buffer)
 {
   GstWildmidi *wildmidi;
 
-  wildmidi = GST_WILDMIDI (parent);
+  wildmidi = GST_WILDMIDI (GST_PAD_PARENT (sinkpad));
 
   /* push stuff in the adapter, we will start doing something in the sink event
    * handler when we get EOS */
@@ -855,7 +821,7 @@ gst_wildmidi_loop (GstPad * sinkpad)
       ret =
           gst_pad_pull_range (wildmidi->sinkpad, wildmidi->offset, -1, &buffer);
 
-      if (ret == GST_FLOW_EOS) {
+      if (ret == GST_FLOW_UNEXPECTED) {
         GST_DEBUG_OBJECT (wildmidi, "Song loaded");
         wildmidi->state = GST_WILDMIDI_STATE_PARSE;
       } else if (ret != GST_FLOW_OK) {
@@ -865,7 +831,7 @@ gst_wildmidi_loop (GstPad * sinkpad)
       } else {
         GST_DEBUG_OBJECT (wildmidi, "pushing buffer");
         gst_adapter_push (wildmidi->adapter, buffer);
-        wildmidi->offset += gst_buffer_get_size (buffer);
+        wildmidi->offset += GST_BUFFER_SIZE (buffer);
       }
       break;
     }
@@ -892,11 +858,11 @@ pause:
 
     GST_DEBUG_OBJECT (wildmidi, "pausing task, reason %s", reason);
     gst_pad_pause_task (sinkpad);
-    if (ret == GST_FLOW_EOS) {
+    if (ret == GST_FLOW_UNEXPECTED) {
       /* perform EOS logic */
       event = gst_event_new_eos ();
       gst_pad_push_event (wildmidi->srcpad, event);
-    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_EOS) {
+    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_UNEXPECTED) {
       event = gst_event_new_eos ();
       /* for fatal errors we post an error message, post the error
        * first so the app knows about the error first. */
