@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 
@@ -53,7 +53,6 @@ struct _GstZBar
 
   /* properties */
   gboolean message;
-  gboolean attach_frame;
   gboolean cache;
 
   /* internals */
@@ -67,8 +66,6 @@ struct _GstZBarClass
 
 GType gst_zbar_get_type(void);
 
-GST_ELEMENT_REGISTER_DECLARE (zbar);
-
 G_END_DECLS
 
 #endif /* __GST_VIDEO_ZBAR_H__ */
