@@ -39,8 +39,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef __GST_BURN_H__
@@ -53,6 +53,7 @@
 
 G_BEGIN_DECLS
 
+/* #defines don't like whitespacey bits */
 #define GST_TYPE_BURN \
   (gst_burn_get_type())
 #define GST_BURN(obj) \
@@ -72,7 +73,9 @@ struct _GstBurn
   GstVideoFilter videofilter;
 
   /* < private > */
+
   gint adjustment;
+  gboolean silent;
 };
 
 struct _GstBurnClass
