@@ -16,26 +16,26 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /**
  * SECTION:element-jifmux
- * @title: jifmux
  * @short_description: JPEG interchange format writer
  *
  * Writes a JPEG image as JPEG/EXIF or JPEG/JFIF including various metadata. The
  * jpeg image received on the sink pad should be minimal (e.g. should not
  * contain metadata already).
  *
- * ## Example launch line
+ * <refsect2>
+ * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 -v videotestsrc num-buffers=1 ! jpegenc ! jifmux ! filesink location=...
+ * gst-launch -v videotestsrc num-buffers=1 ! jpegenc ! jifmux ! filesink location=...
  * ]|
  * The above pipeline renders a frame, encodes to jpeg, adds metadata and writes
  * it to disk.
- *
+ * </refsect2>
  */
 /*
 jpeg interchange format:
@@ -46,8 +46,8 @@ scan header : {DAC,DHT},DRI,SOS
 file trailer: EOI
 
 tests:
-gst-launch-1.0 videotestsrc num-buffers=1 ! jpegenc ! jifmux ! filesink location=test1.jpeg
-gst-launch-1.0 videotestsrc num-buffers=1 ! jpegenc ! taginject tags="comment=test image" ! jifmux ! filesink location=test2.jpeg
+gst-launch videotestsrc num-buffers=1 ! jpegenc ! jifmux ! filesink location=test1.jpeg
+gst-launch videotestsrc num-buffers=1 ! jpegenc ! taginject tags="comment=\"test image\"" ! jifmux ! filesink location=test2.jpeg
 */
 
 #ifdef HAVE_CONFIG_H
@@ -57,7 +57,6 @@ gst-launch-1.0 videotestsrc num-buffers=1 ! jpegenc ! taginject tags="comment=te
 #include <string.h>
 #include <gst/base/gstbytereader.h>
 #include <gst/base/gstbytewriter.h>
-#include <gst/codecparsers/gstjpegparser.h>
 #include <gst/tag/tag.h>
 #include <gst/tag/xmpwriter.h>
 
@@ -96,6 +95,16 @@ typedef struct _GstJifMuxMarker
   gboolean owned;
 } GstJifMuxMarker;
 
+struct _GstJifMuxPrivate
+{
+  GstPad *srcpad;
+
+  /* list of GstJifMuxMarker */
+  GList *markers;
+  guint scan_size;
+  const guint8 *scan_data;
+};
+
 static void gst_jif_mux_finalize (GObject * object);
 
 static void gst_jif_mux_reset (GstJifMux * self);
@@ -111,8 +120,6 @@ static GstStateChangeReturn gst_jif_mux_change_state (GstElement * element,
 G_DEFINE_TYPE_WITH_CODE (GstJifMux, gst_jif_mux, GST_TYPE_ELEMENT,
     G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_SETTER, NULL);
     G_IMPLEMENT_INTERFACE (GST_TYPE_TAG_XMP_WRITER, NULL));
-GST_ELEMENT_REGISTER_DEFINE (jifmux, "jifmux", GST_RANK_SECONDARY,
-    GST_TYPE_JIF_MUX);
 
 static void
 gst_jif_mux_class_init (GstJifMuxClass * klass)
@@ -123,14 +130,16 @@ gst_jif_mux_class_init (GstJifMuxClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
+  g_type_class_add_private (gobject_class, sizeof (GstJifMuxPrivate));
+
   gobject_class->finalize = gst_jif_mux_finalize;
 
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_jif_mux_change_state);
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_jif_mux_src_pad_template);
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &gst_jif_mux_sink_pad_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_jif_mux_src_pad_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_jif_mux_sink_pad_template));
 
   gst_element_class_set_static_metadata (gstelement_class,
       "JPEG stream muxer",
@@ -147,6 +156,9 @@ gst_jif_mux_init (GstJifMux * self)
 {
   GstPad *sinkpad;
 
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GST_TYPE_JIF_MUX,
+      GstJifMuxPrivate);
+
   /* create the sink and src pads */
   sinkpad = gst_pad_new_from_static_template (&gst_jif_mux_sink_pad_template,
       "sink");
@@ -155,9 +167,9 @@ gst_jif_mux_init (GstJifMux * self)
       GST_DEBUG_FUNCPTR (gst_jif_mux_sink_event));
   gst_element_add_pad (GST_ELEMENT (self), sinkpad);
 
-  self->srcpad =
+  self->priv->srcpad =
       gst_pad_new_from_static_template (&gst_jif_mux_src_pad_template, "src");
-  gst_element_add_pad (GST_ELEMENT (self), self->srcpad);
+  gst_element_add_pad (GST_ELEMENT (self), self->priv->srcpad);
 }
 
 static void
@@ -181,7 +193,7 @@ gst_jif_mux_sink_setcaps (GstJifMux * self, GstCaps * caps)
     /* FIXME: do we want to switch it like this or use a gobject property ? */
   }
 
-  return gst_pad_set_caps (self->srcpad, caps);
+  return gst_pad_set_caps (self->priv->srcpad, caps);
 }
 
 static gboolean
@@ -225,7 +237,7 @@ gst_jif_mux_marker_free (GstJifMuxMarker * m)
   if (m->owned)
     g_free ((gpointer) m->data);
 
-  g_free (m);
+  g_slice_free (GstJifMuxMarker, m);
 }
 
 static void
@@ -234,19 +246,19 @@ gst_jif_mux_reset (GstJifMux * self)
   GList *node;
   GstJifMuxMarker *m;
 
-  for (node = self->markers; node; node = g_list_next (node)) {
+  for (node = self->priv->markers; node; node = g_list_next (node)) {
     m = (GstJifMuxMarker *) node->data;
     gst_jif_mux_marker_free (m);
   }
-  g_list_free (self->markers);
-  self->markers = NULL;
+  g_list_free (self->priv->markers);
+  self->priv->markers = NULL;
 }
 
 static GstJifMuxMarker *
 gst_jif_mux_new_marker (guint8 marker, guint16 size, const guint8 * data,
     gboolean owned)
 {
-  GstJifMuxMarker *m = g_new (GstJifMuxMarker, 1);
+  GstJifMuxMarker *m = g_slice_new (GstJifMuxMarker);
 
   m->marker = marker;
   m->size = size;
@@ -282,24 +294,25 @@ gst_jif_mux_parse_image (GstJifMux * self, GstBuffer * buf)
       goto error;
 
     switch (marker) {
-      case GST_JPEG_MARKER_RST0:
-      case GST_JPEG_MARKER_RST1:
-      case GST_JPEG_MARKER_RST2:
-      case GST_JPEG_MARKER_RST3:
-      case GST_JPEG_MARKER_RST4:
-      case GST_JPEG_MARKER_RST5:
-      case GST_JPEG_MARKER_RST6:
-      case GST_JPEG_MARKER_RST7:
-      case GST_JPEG_MARKER_SOI:
+      case RST0:
+      case RST1:
+      case RST2:
+      case RST3:
+      case RST4:
+      case RST5:
+      case RST6:
+      case RST7:
+      case SOI:
         GST_DEBUG_OBJECT (self, "marker = %x", marker);
         m = gst_jif_mux_new_marker (marker, 0, NULL, FALSE);
-        self->markers = g_list_prepend (self->markers, m);
+        self->priv->markers = g_list_prepend (self->priv->markers, m);
         break;
-      case GST_JPEG_MARKER_EOI:
+      case EOI:
         GST_DEBUG_OBJECT (self, "marker = %x", marker);
         m = gst_jif_mux_new_marker (marker, 0, NULL, FALSE);
-        self->markers = g_list_prepend (self->markers, m);
+        self->priv->markers = g_list_prepend (self->priv->markers, m);
         goto done;
+        break;
       default:
         if (!gst_byte_reader_get_uint16_be (&reader, &size))
           goto error;
@@ -307,21 +320,20 @@ gst_jif_mux_parse_image (GstJifMux * self, GstBuffer * buf)
           goto error;
 
         m = gst_jif_mux_new_marker (marker, size - 2, data, FALSE);
-        self->markers = g_list_prepend (self->markers, m);
+        self->priv->markers = g_list_prepend (self->priv->markers, m);
 
         GST_DEBUG_OBJECT (self, "marker = %2x, size = %u", marker, size);
         break;
     }
 
-    if (marker == GST_JPEG_MARKER_SOS) {
+    if (marker == SOS) {
       gint eoi_pos = -1;
       gint i;
 
       /* search the last 5 bytes for the EOI marker */
       g_assert (map.size >= 5);
       for (i = 5; i >= 2; i--) {
-        if (map.data[map.size - i] == 0xFF
-            && map.data[map.size - i + 1] == GST_JPEG_MARKER_EOI) {
+        if (map.data[map.size - i] == 0xFF && map.data[map.size - i + 1] == EOI) {
           eoi_pos = map.size - i;
           break;
         }
@@ -332,12 +344,12 @@ gst_jif_mux_parse_image (GstJifMux * self, GstBuffer * buf)
       }
 
       /* remaining size except EOI is scan data */
-      self->scan_size = eoi_pos - gst_byte_reader_get_pos (&reader);
-      if (!gst_byte_reader_get_data (&reader, self->scan_size,
-              &self->scan_data))
+      self->priv->scan_size = eoi_pos - gst_byte_reader_get_pos (&reader);
+      if (!gst_byte_reader_get_data (&reader, self->priv->scan_size,
+              &self->priv->scan_data))
         goto error;
 
-      GST_DEBUG_OBJECT (self, "scan data, size = %u", self->scan_size);
+      GST_DEBUG_OBJECT (self, "scan data, size = %u", self->priv->scan_size);
     }
 
     if (!gst_byte_reader_peek_uint8 (&reader, &marker))
@@ -347,7 +359,7 @@ gst_jif_mux_parse_image (GstJifMux * self, GstBuffer * buf)
       gst_byte_reader_get_pos (&reader), (guint) map.size);
 
 done:
-  self->markers = g_list_reverse (self->markers);
+  self->priv->markers = g_list_reverse (self->priv->markers);
   gst_buffer_unmap (buf, &map);
 
   return TRUE;
@@ -385,12 +397,12 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
    */
 
   /* find some reference points where we insert before/after */
-  file_hdr = self->markers;
-  for (node = self->markers; node; node = g_list_next (node)) {
+  file_hdr = self->priv->markers;
+  for (node = self->priv->markers; node; node = g_list_next (node)) {
     m = (GstJifMuxMarker *) node->data;
 
     switch (m->marker) {
-      case GST_JPEG_MARKER_APP0:
+      case APP0:
         if (m->size > 5 && !memcmp (m->data, "JFIF\0", 5)) {
           GST_DEBUG_OBJECT (self, "found APP0 JFIF");
           colorspace |= COLORSPACE_GRAYSCALE | COLORSPACE_YUV;
@@ -398,7 +410,7 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
             app0_jfif = node;
         }
         break;
-      case GST_JPEG_MARKER_APP1:
+      case APP1:
         if (m->size > 6 && (!memcmp (m->data, "EXIF\0\0", 6) ||
                 !memcmp (m->data, "Exif\0\0", 6))) {
           GST_DEBUG_OBJECT (self, "found APP1 EXIF");
@@ -411,7 +423,7 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
             app1_xmp = node;
         }
         break;
-      case GST_JPEG_MARKER_APP14:
+      case APP14:
         /* check if this contains RGB */
         /*
          * This marker should have:
@@ -443,32 +455,32 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
         }
 
         break;
-      case GST_JPEG_MARKER_COM:
+      case COM:
         GST_INFO_OBJECT (self, "found COM, will be replaced");
         if (!com)
           com = node;
         break;
-      case GST_JPEG_MARKER_DQT:
-      case GST_JPEG_MARKER_SOF0:
-      case GST_JPEG_MARKER_SOF1:
-      case GST_JPEG_MARKER_SOF2:
-      case GST_JPEG_MARKER_SOF3:
-      case GST_JPEG_MARKER_SOF5:
-      case GST_JPEG_MARKER_SOF6:
-      case GST_JPEG_MARKER_SOF7:
-      case GST_JPEG_MARKER_SOF9:
-      case GST_JPEG_MARKER_SOF10:
-      case GST_JPEG_MARKER_SOF11:
-      case GST_JPEG_MARKER_SOF13:
-      case GST_JPEG_MARKER_SOF14:
-      case GST_JPEG_MARKER_SOF15:
+      case DQT:
+      case SOF0:
+      case SOF1:
+      case SOF2:
+      case SOF3:
+      case SOF5:
+      case SOF6:
+      case SOF7:
+      case SOF9:
+      case SOF10:
+      case SOF11:
+      case SOF13:
+      case SOF14:
+      case SOF15:
         if (!frame_hdr)
           frame_hdr = node;
         break;
-      case GST_JPEG_MARKER_DAC:
-      case GST_JPEG_MARKER_DHT:
-      case GST_JPEG_MARKER_DRI:
-      case GST_JPEG_MARKER_SOS:
+      case DAC:
+      case DHT:
+      case DRI:
+      case SOS:
         if (!scan_hdr)
           scan_hdr = node;
         break;
@@ -479,7 +491,6 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
   /* check if we don't have JFIF APP0 */
   if (!app0_jfif && (colorspace & (COLORSPACE_GRAYSCALE | COLORSPACE_YUV))) {
     /* build jfif header */
-    /* *INDENT-OFF* */
     static const struct
     {
       gchar id[5];
@@ -487,20 +498,17 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
       guint8 du;
       guint8 xd[2], yd[2];
       guint8 tw, th;
-    } jfif_data = { "JFIF",
-        { 1, 2 },
-        0,
-        { 0, 1 }, /* FIXME: check pixel-aspect from caps */
-        { 0, 1 },
-        0, 0
-    };
-    /* *INDENT-ON* */
-
-    m = gst_jif_mux_new_marker (GST_JPEG_MARKER_APP0, sizeof (jfif_data),
+    } jfif_data = {
+      "JFIF", {
+      1, 2}, 0, {
+      0, 1},                    /* FIXME: check pixel-aspect from caps */
+      {
+    0, 1}, 0, 0};
+    m = gst_jif_mux_new_marker (APP0, sizeof (jfif_data),
         (const guint8 *) &jfif_data, FALSE);
     /* insert into self->markers list */
-    self->markers = g_list_insert (self->markers, m, 1);
-    app0_jfif = g_list_nth (self->markers, 1);
+    self->priv->markers = g_list_insert (self->priv->markers, m, 1);
+    app0_jfif = g_list_nth (self->priv->markers, 1);
   }
   /* else */
   /* remove JFIF if exists */
@@ -544,8 +552,7 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
       data = g_malloc0 (exif_size + 6);
       memcpy (data, "Exif", 4);
       gst_buffer_extract (exif_data, 0, data + 6, exif_size);
-      m = gst_jif_mux_new_marker (GST_JPEG_MARKER_APP1, exif_size + 6, data,
-          TRUE);
+      m = gst_jif_mux_new_marker (APP1, exif_size + 6, data, TRUE);
       gst_buffer_unref (exif_data);
 
       if (app1_exif) {
@@ -557,11 +564,12 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
           pos = app0_jfif;
         pos = g_list_next (pos);
 
-        self->markers = g_list_insert_before (self->markers, pos, m);
+        self->priv->markers =
+            g_list_insert_before (self->priv->markers, pos, m);
         if (pos) {
           app1_exif = g_list_previous (pos);
         } else {
-          app1_exif = g_list_last (self->markers);
+          app1_exif = g_list_last (self->priv->markers);
         }
       }
       modified = TRUE;
@@ -581,7 +589,7 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
     data = g_malloc (size + 29);
     memcpy (data, "http://ns.adobe.com/xap/1.0/\0", 29);
     gst_buffer_extract (xmp_data, 0, &data[29], size);
-    m = gst_jif_mux_new_marker (GST_JPEG_MARKER_APP1, size + 29, data, TRUE);
+    m = gst_jif_mux_new_marker (APP1, size + 29, data, TRUE);
 
     /*
      * Replace the old xmp marker and not add a new one.
@@ -600,7 +608,7 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
         pos = app0_jfif;
       pos = g_list_next (pos);
 
-      self->markers = g_list_insert_before (self->markers, pos, m);
+      self->priv->markers = g_list_insert_before (self->priv->markers, pos, m);
 
     }
     gst_buffer_unref (xmp_data);
@@ -615,11 +623,12 @@ gst_jif_mux_mangle_markers (GstJifMux * self)
   if (str) {
     GST_DEBUG_OBJECT (self, "set COM marker to '%s'", str);
     /* insert new marker into self->markers list */
-    m = gst_jif_mux_new_marker (GST_JPEG_MARKER_COM, strlen (str) + 1,
-        (const guint8 *) str, TRUE);
+    m = gst_jif_mux_new_marker (COM, strlen (str) + 1, (const guint8 *) str,
+        TRUE);
     /* FIXME: if we have one already, replace */
     /* this should go before SOS, maybe at the end of file-header */
-    self->markers = g_list_insert_before (self->markers, frame_hdr, m);
+    self->priv->markers = g_list_insert_before (self->priv->markers,
+        frame_hdr, m);
 
     modified = TRUE;
   }
@@ -637,12 +646,12 @@ gst_jif_mux_recombine_image (GstJifMux * self, GstBuffer ** new_buf,
   GstByteWriter *writer;
   GstJifMuxMarker *m;
   GList *node;
-  guint size = self->scan_size;
+  guint size = self->priv->scan_size;
   gboolean writer_status = TRUE;
   GstMapInfo map;
 
   /* iterate list and collect size */
-  for (node = self->markers; node; node = g_list_next (node)) {
+  for (node = self->priv->markers; node; node = g_list_next (node)) {
     m = (GstJifMuxMarker *) node->data;
 
     /* some markers like e.g. SOI are empty */
@@ -666,7 +675,8 @@ gst_jif_mux_recombine_image (GstJifMux * self, GstBuffer ** new_buf,
   gst_buffer_map (buf, &map, GST_MAP_WRITE);
   writer = gst_byte_writer_new_with_data (map.data, map.size, TRUE);
 
-  for (node = self->markers; node && writer_status; node = g_list_next (node)) {
+  for (node = self->priv->markers; node && writer_status;
+      node = g_list_next (node)) {
     m = (GstJifMuxMarker *) node->data;
 
     writer_status &= gst_byte_writer_put_uint8 (writer, 0xff);
@@ -679,10 +689,11 @@ gst_jif_mux_recombine_image (GstJifMux * self, GstBuffer ** new_buf,
       writer_status &= gst_byte_writer_put_data (writer, m->data, m->size);
     }
 
-    if (m->marker == GST_JPEG_MARKER_SOS) {
-      GST_DEBUG_OBJECT (self, "scan data, size = %u", self->scan_size);
+    if (m->marker == SOS) {
+      GST_DEBUG_OBJECT (self, "scan data, size = %u", self->priv->scan_size);
       writer_status &=
-          gst_byte_writer_put_data (writer, self->scan_data, self->scan_size);
+          gst_byte_writer_put_data (writer, self->priv->scan_data,
+          self->priv->scan_size);
     }
   }
   gst_buffer_unmap (buf, &map);
@@ -726,7 +737,7 @@ gst_jif_mux_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   gst_jif_mux_reset (self);
 
   if (fret == GST_FLOW_OK) {
-    fret = gst_pad_push (self->srcpad, buf);
+    fret = gst_pad_push (self->priv->srcpad, buf);
   }
   return fret;
 }
