@@ -22,8 +22,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 
@@ -57,34 +57,20 @@ typedef struct _MpegTSBaseProgram MpegTSBaseProgram;
 
 struct _MpegTSBaseStream
 {
-  guint16             pid;
-  guint8              stream_type;
-
-  /* Content of the registration descriptor (if present) */
-  guint32             registration_id;
-
-  GstMpegtsPMTStream *stream;
-  GstStream          *stream_object;
-  gchar              *stream_id;
+  guint16 pid;
+  guint8 stream_type;
+  GstStructure* stream_info;
 };
 
 struct _MpegTSBaseProgram
 {
-  gint                program_number;
-  guint16             pmt_pid;
-  guint16             pcr_pid;
-
-  /* Content of the registration descriptor (if present) */
-  guint32             registration_id;
-
-  GstMpegtsSection   *section;
-  const GstMpegtsPMT *pmt;
-
-  MpegTSBaseStream  **streams;
-  GList              *stream_list;
-  gint                patcount;
-
-  GstStreamCollection *collection;
+  gint program_number;
+  guint16 pmt_pid;
+  guint16 pcr_pid;
+  GstStructure *pmt_info;
+  MpegTSBaseStream **streams;
+  GList *stream_list;
+  gint patcount;
 
   /* Pending Tags for the program */
   GstTagList *tags;
@@ -124,7 +110,7 @@ struct _MpegTSBase {
    * accessed from the application thread and the streaming thread */
   GHashTable *programs;
 
-  GPtrArray  *pat;
+  GstStructure *pat;
   MpegTSPacketizer2 *packetizer;
 
   /* arrays that say whether a pid is a known psi pid or a pes pid */
@@ -145,30 +131,13 @@ struct _MpegTSBase {
   /* Whether we saw a PAT yet */
   gboolean seen_pat;
 
+  /* Whether upstream is live or not */
+  gboolean upstream_live;
+  /* Whether we queried the upstream latency or not */
+  gboolean queried_latency;
+
   /* Upstream segment */
   GstSegment segment;
-
-  /* Downstream segment, for use by sub-classes */
-  GstSegment out_segment;
-
-  /* Last received seek event seqnum (default GST_SEQNUM_INVALID) */
-  guint last_seek_seqnum;
-
-  /* Whether to parse private section or not */
-  gboolean parse_private_sections;
-
-  /* Whether to push data and/or sections to subclasses */
-  gboolean push_data;
-  gboolean push_section;
-  gboolean push_unknown;
-
-  /* Whether the parent bin is streams-aware, meaning we can
-   * add/remove streams at any point in time */
-  gboolean streams_aware;
-
-  /* Do not use the PCR stream for timestamp calculation. Useful for
-   * streams with broken/invalid PCR streams. */
-  gboolean ignore_pcr;
 };
 
 struct _MpegTSBaseClass {
@@ -176,8 +145,7 @@ struct _MpegTSBaseClass {
 
   /* Virtual methods */
   void (*reset) (MpegTSBase *base);
-  GstFlowReturn (*push) (MpegTSBase *base, MpegTSPacketizerPacket *packet, GstMpegtsSection * section);
-  void (*inspect_packet) (MpegTSBase *base, MpegTSPacketizerPacket *packet);
+  GstFlowReturn (*push) (MpegTSBase *base, MpegTSPacketizerPacket *packet, MpegTSPacketizerSection * section);
   /* takes ownership of @event */
   gboolean (*push_event) (MpegTSBase *base, GstEvent * event);
 
@@ -185,14 +153,9 @@ struct _MpegTSBaseClass {
   void (*program_started) (MpegTSBase *base, MpegTSBaseProgram *program);
   /* program_stopped gets called when pat no longer has program's pmt */
   void (*program_stopped) (MpegTSBase *base, MpegTSBaseProgram *program);
-  void (*update_program) (MpegTSBase *base, MpegTSBaseProgram *program);
-  /* Whether mpegtbase can deactivate/free a program or whether the subclass will do it
-   * If the subclass responds TRUE, it should call mpegts_base_deactivate_and_free_program()
-   * when it wants to remove it */
-  gboolean (*can_remove_program) (MpegTSBase *base, MpegTSBaseProgram *program);
 
   /* stream_added is called whenever a new stream has been identified */
-  gboolean (*stream_added) (MpegTSBase *base, MpegTSBaseStream *stream, MpegTSBaseProgram *program);
+  void (*stream_added) (MpegTSBase *base, MpegTSBaseStream *stream, MpegTSBaseProgram *program);
   /* stream_removed is called whenever a stream is no longer referenced */
   void (*stream_removed) (MpegTSBase *base, MpegTSBaseStream *stream);
 
@@ -202,16 +165,11 @@ struct _MpegTSBaseClass {
   /* seek is called to wait for seeking */
   GstFlowReturn (*seek) (MpegTSBase * base, GstEvent * event);
 
-  /* Drain all currently pending data */
-  GstFlowReturn (*drain) (MpegTSBase * base);
-
-  /* flush all streams
-   * The hard inicator is used to flush completely on FLUSH_STOP events
-   * or partially in pull mode seeks of tsdemux */
-  void (*flush) (MpegTSBase * base, gboolean hard);
+  /* flush all streams */
+  void (*flush) (MpegTSBase * base);
 
   /* Notifies subclasses input buffer has been handled */
-  GstFlowReturn (*input_done) (MpegTSBase *base);
+  GstFlowReturn (*input_done) (MpegTSBase *base, GstBuffer *buffer);
 
   /* signals */
   void (*pat_info) (GstStructure *pat);
@@ -219,9 +177,6 @@ struct _MpegTSBaseClass {
   void (*nit_info) (GstStructure *nit);
   void (*sdt_info) (GstStructure *sdt);
   void (*eit_info) (GstStructure *eit);
-
-  /* takes ownership of @query */
-  gboolean (*sink_query) (MpegTSBase *base, GstQuery * query);
 };
 
 #define MPEGTS_BIT_SET(field, offs)    ((field)[(offs) >> 3] |=  (1 << ((offs) & 0x7)))
@@ -233,16 +188,19 @@ G_GNUC_INTERNAL GType mpegts_base_get_type(void);
 G_GNUC_INTERNAL MpegTSBaseProgram *mpegts_base_get_program (MpegTSBase * base, gint program_number);
 G_GNUC_INTERNAL MpegTSBaseProgram *mpegts_base_add_program (MpegTSBase * base, gint program_number, guint16 pmt_pid);
 
-G_GNUC_INTERNAL const GstMpegtsDescriptor *mpegts_get_descriptor_from_stream (MpegTSBaseStream * stream, guint8 tag);
-G_GNUC_INTERNAL const GstMpegtsDescriptor *mpegts_get_descriptor_from_program (MpegTSBaseProgram * program, guint8 tag);
+G_GNUC_INTERNAL guint8 *mpegts_get_descriptor_from_stream (MpegTSBaseStream * stream, guint8 tag);
+G_GNUC_INTERNAL guint8 *mpegts_get_descriptor_from_program (MpegTSBaseProgram * program, guint8 tag);
 
 G_GNUC_INTERNAL gboolean
 mpegts_base_handle_seek_event(MpegTSBase * base, GstPad * pad, GstEvent * event);
 
 G_GNUC_INTERNAL gboolean gst_mpegtsbase_plugin_init (GstPlugin * plugin);
 
-G_GNUC_INTERNAL void mpegts_base_deactivate_and_free_program (MpegTSBase *base, MpegTSBaseProgram *program);
+G_GNUC_INTERNAL gboolean mpegts_base_handle_psi (MpegTSBase * base, MpegTSPacketizerSection * section);
 
+G_GNUC_INTERNAL void mpegts_base_program_remove_stream (MpegTSBase * base, MpegTSBaseProgram * program, guint16 pid);
+
+G_GNUC_INTERNAL void mpegts_base_remove_program(MpegTSBase *base, gint program_number);
 G_END_DECLS
 
 #endif /* GST_MPEG_TS_BASE_H */
