@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +37,8 @@ GST_DEBUG_CATEGORY_STATIC (pes_parser_debug);
  * @data: data to parse (starting from, and including, the sync code)
  * @length: size of @data in bytes
  * @res: PESHeader to fill (only valid with #PES_PARSING_OK.
+ * @offset: Offset in @data to the data to parse. If #PES_PARSING_OK, offset to
+ *	    first byte of data after the header.
  *
  * Parses the mpeg-ts PES header located in @data into the @res.
  *
@@ -45,7 +47,8 @@ GST_DEBUG_CATEGORY_STATIC (pes_parser_debug);
  * is needed to properly parse the header.
  */
 PESParsingResult
-mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
+mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res,
+    gint * offset)
 {
   PESParsingResult ret = PES_PARSING_NEED_MORE;
   gsize origlength = length;
@@ -53,7 +56,12 @@ mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
   guint32 val32;
   guint8 val8, flags;
 
-  g_assert (res != NULL);
+  g_return_val_if_fail (res != NULL, PES_PARSING_BAD);
+  g_return_val_if_fail (offset != NULL, PES_PARSING_BAD);
+  g_return_val_if_fail (*offset < length, PES_PARSING_BAD);
+
+  data += *offset;
+  length -= *offset;
 
   /* The smallest valid PES header is 6 bytes (prefix + stream_id + length) */
   if (G_UNLIKELY (length < 6))
@@ -105,12 +113,6 @@ mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
   res->flags = val8 & 0xf;
 
   GST_LOG ("scrambling_control 0x%0x", res->scrambling_control);
-  GST_LOG ("flags_1: %s%s%s%s%s",
-      val8 & PES_FLAG_PRIORITY ? "priority " : "",
-      val8 & PES_FLAG_DATA_ALIGNMENT ? "data_alignment " : "",
-      val8 & PES_FLAG_COPYRIGHT ? "copyright " : "",
-      val8 & PES_FLAG_ORIGINAL_OR_COPY ? "original_or_copy " : "",
-      val8 & 0x0f ? "" : "<none>");
 
   /* PTS_DTS_flags                    2
    * ESCR_flag                        1
@@ -120,15 +122,7 @@ mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
    * PES_CRC_flag                     1
    * PES_extension_flag               1*/
   flags = *data++;
-  GST_LOG ("flags_2: %s%s%s%s%s%s%s%s%s",
-      flags & 0x80 ? "PTS " : "",
-      flags & 0x40 ? "DTS " : "",
-      flags & 0x20 ? "ESCR" : "",
-      flags & 0x10 ? "ES_rate " : "",
-      flags & 0x08 ? "DSM_trick_mode " : "",
-      flags & 0x04 ? "additional_copy_info " : "",
-      flags & 0x02 ? "CRC " : "",
-      flags & 0x01 ? "extension " : "", flags ? "" : "<none>");
+  GST_DEBUG ("PES_flag 0x%02x", flags);
 
   /* PES_header_data_length           8 */
   res->header_size = *data++;
@@ -210,7 +204,7 @@ mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
       case PES_TRICK_MODE_FAST_REVERSE:
         res->intra_slice_refresh = (val8 >> 2) & 0x1;
         res->frequency_truncation = val8 & 0x3;
-        G_GNUC_FALLTHROUGH;
+        /* passthrough */
       case PES_TRICK_MODE_FREEZE_FRAME:
         res->field_id = (val8 >> 3) & 0x3;
         break;
@@ -257,12 +251,7 @@ mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
   /* PES extension */
   flags = *data++;
   length -= 1;
-  GST_DEBUG ("PES_extension_flag: %s%s%s%s%s%s",
-      flags & 0x80 ? "PES_private_data " : "",
-      flags & 0x40 ? "pack_header_field " : "",
-      flags & 0x20 ? "program_packet_sequence_counter " : "",
-      flags & 0x10 ? "P-STD_buffer " : "",
-      flags & 0x01 ? "PES_extension_flag_2" : "", flags & 0xf1 ? "" : "<none>");
+  GST_DEBUG ("PES_extension_flag 0x%02x", flags);
 
   if (flags & 0x80) {
     /* PES_private data */
@@ -298,6 +287,7 @@ mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
       goto need_more_data;
 
     val8 = *data++;
+    /* GRMBL, this is most often wrong */
     if (G_UNLIKELY ((val8 & 0x80) != 0x80))
       goto bad_sequence_marker1;
     res->program_packet_sequence_counter = val8 & 0x7f;
@@ -305,6 +295,7 @@ mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
         res->program_packet_sequence_counter);
 
     val8 = *data++;
+    /* GRMBL, this is most often wrong */
     if (G_UNLIKELY ((val8 & 0x80) != 0x80))
       goto bad_sequence_marker2;
     res->MPEG1_MPEG2_identifier = (val8 >> 6) & 0x1;
@@ -315,74 +306,51 @@ mpegts_parse_pes_header (const guint8 * data, gsize length, PESHeader * res)
   }
 
   if (flags & 0x10) {
-    /* P-STD
-     * '01'               :  2 bits
-     * P-STD_buffer_scale :  1 bit
-     * P-STD_buffer_size  : 13 bits
-     * */
+    /* P-STD */
     if (G_UNLIKELY (length < 2))
       goto need_more_data;
     val8 = *data;
     if (G_UNLIKELY ((val8 & 0xc0) != 0x40))
       goto bad_P_STD_marker;
-    /* If P-STD_buffer_scale is 0
-     *   multiply by 128 (i.e. << 7),
-     * else
-     *   multiply by 1024 (i.e. << 10)
-     */
     res->P_STD_buffer_size =
-        (GST_READ_UINT16_BE (data) & 0x1fff) << ((val8 & 0x20) ? 10 : 7);
+        (GST_READ_UINT16_BE (data) & 0x1fff) << (val8 & 0x20) ? 10 : 7;
     GST_LOG ("P_STD_buffer_size : %d", res->P_STD_buffer_size);
     data += 2;
     length -= 2;
   }
 
-  /* jump if we don't have a PES 2nd extension */
-  if (!(flags & 0x01))
-    goto stuffing_byte;
+  if (flags & 0x01) {
+    /* Extension flag 2 */
+    if (G_UNLIKELY (length < 1))
+      goto need_more_data;
 
-  /* Extension flag 2 */
-  if (G_UNLIKELY (length < 1))
-    goto need_more_data;
+    val8 = *data++;
+    length -= 1;
 
-  val8 = *data++;
-  length -= 1;
+    if (!(val8 & 0x80))
+      goto bad_extension_marker_2;
 
-  if (!(val8 & 0x80))
-    goto bad_extension_marker_2;
+    res->extension_field_length = val8 & 0x7f;
+    if (G_UNLIKELY (length < res->extension_field_length + 1))
+      goto need_more_data;
 
-  res->extension_field_length = val8 & 0x7f;
+    GST_LOG ("extension_field_length : %" G_GSSIZE_FORMAT,
+        res->extension_field_length);
 
-  /* Skip empty extensions */
-  if (G_UNLIKELY (res->extension_field_length == 0))
-    goto stuffing_byte;
-
-  if (G_UNLIKELY (length < res->extension_field_length))
-    goto need_more_data;
-
-  flags = *data++;
-  res->extension_field_length -= 1;
-
-  if (!(flags & 0x80)) {
-    /* Only valid if stream_id_extension_flag == 0x0 */
-    res->stream_id_extension = flags;
-    GST_LOG ("stream_id_extension : 0x%02x", res->stream_id_extension);
-  } else if (!(flags & 0x01)) {
-    /* Skip broken streams (that use stream_id_extension with highest bit set
-     * for example ...) */
-    if (G_UNLIKELY (res->extension_field_length < 5))
-      goto stuffing_byte;
-
-    GST_LOG ("TREF field present");
-    data += 5;
-    res->extension_field_length -= 5;
-  }
-
-  /* Extension field data */
-  if (res->extension_field_length) {
-    res->stream_id_extension_data = data;
-    GST_MEMDUMP ("stream_id_extension_data",
-        res->stream_id_extension_data, res->extension_field_length);
+    if (res->extension_field_length) {
+      flags = *data++;
+      /* Only valid if stream_id_extension_flag == 0x0 */
+      if (!(flags & 0x80)) {
+        res->stream_id_extension = flags & 0x7f;
+        GST_LOG ("stream_id_extension : 0x%02x", res->stream_id_extension);
+        res->stream_id_extension_data = data;
+        GST_MEMDUMP ("stream_id_extension_data",
+            res->stream_id_extension_data, res->extension_field_length);
+      } else
+        GST_WARNING ("What are we meant to do ??");
+      data += res->extension_field_length;
+    }
+    length -= res->extension_field_length + 1;
   }
 
 stuffing_byte:
@@ -391,10 +359,11 @@ stuffing_byte:
   length = origlength - res->header_size;
 
 done_parsing:
-  GST_DEBUG ("origlength:%" G_GSIZE_FORMAT ", length:%" G_GSIZE_FORMAT,
+  GST_DEBUG ("origlength:%" G_GSSIZE_FORMAT ", length:%" G_GSSIZE_FORMAT,
       origlength, length);
 
   res->header_size = origlength - length;
+  *offset += res->header_size;
   ret = PES_PARSING_OK;
 
   return ret;
