@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,10 +27,12 @@
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  gboolean ret = FALSE;
+  gboolean ret;
 
-  ret |= GST_ELEMENT_REGISTER (pcapparse, plugin);
-  ret |= GST_ELEMENT_REGISTER (irtspparse, plugin);
+  ret = gst_element_register (plugin, "pcapparse",
+      GST_RANK_NONE, GST_TYPE_PCAP_PARSE);
+  ret &= gst_element_register (plugin, "irtspparse",
+      GST_RANK_NONE, GST_TYPE_IRTSP_PARSE);
 
   return ret;
 }
@@ -39,4 +41,4 @@ GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     pcapparse,
     "Element parsing raw pcap streams",
-    plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
+    plugin_init, VERSION, "LGPL", "GStreamer", "http://gstreamer.net/")
