@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /* TODO: - Add locking where appropiate
@@ -29,6 +29,10 @@
 
 #include <string.h>
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #include <gst/gst.h>
 #include <gst/base/gstbasetransform.h>
 #include <gst/base/gstadapter.h>
@@ -198,7 +202,6 @@ gst_raw_parse_reset (GstRawParse * rp)
 {
   rp->n_frames = 0;
   rp->discont = TRUE;
-  rp->negotiated = FALSE;
 
   gst_segment_init (&rp->segment, GST_FORMAT_TIME);
   gst_adapter_clear (rp->adapter);
@@ -223,23 +226,6 @@ gst_raw_parse_set_src_caps (GstRawParse * rp)
 
   rp->negotiated = gst_pad_set_caps (rp->srcpad, caps);
 
-  /* if subclass inplement decide_allocation, send an allocation
-   * query, pass result to subclass and let it handle allocation if needed. */
-  if (rp_class->decide_allocation) {
-    GstQuery *query;
-
-    query = gst_query_new_allocation (caps, TRUE);
-    if (!gst_pad_peer_query (rp->srcpad, query)) {
-      /* not a problem, just debug a little */
-      GST_DEBUG_OBJECT (rp, "peer ALLOCATION query failed");
-    }
-
-    rp_class->decide_allocation (rp, query);
-    gst_query_unref (query);
-  }
-
-  gst_caps_unref (caps);
-
   return rp->negotiated;
 }
 
@@ -249,12 +235,10 @@ gst_raw_parse_push_buffer (GstRawParse * rp, GstBuffer * buffer)
   GstFlowReturn ret;
   gint nframes;
   GstRawParseClass *rpclass;
-  gsize size;
 
   rpclass = GST_RAW_PARSE_GET_CLASS (rp);
 
-  size = gst_buffer_get_size (buffer);
-  nframes = size / rp->framesize;
+  nframes = gst_buffer_get_size (buffer) / rp->framesize;
 
   if (rp->segment.rate < 0) {
     rp->n_frames -= nframes;
@@ -275,8 +259,8 @@ gst_raw_parse_push_buffer (GstRawParse * rp, GstBuffer * buffer)
     GST_BUFFER_DURATION (buffer) = GST_CLOCK_TIME_NONE;
   }
 
-  if (rpclass->pre_push_buffer) {
-    rpclass->pre_push_buffer (rp, buffer);
+  if (rpclass->set_buffer_flags) {
+    rpclass->set_buffer_flags (rp, buffer);
   }
 
   if (rp->discont) {
@@ -285,7 +269,7 @@ gst_raw_parse_push_buffer (GstRawParse * rp, GstBuffer * buffer)
   }
 
   if (rp->segment.rate >= 0) {
-    rp->offset += size;
+    rp->offset += gst_buffer_get_size (buffer);
     rp->n_frames += nframes;
   }
 
@@ -305,7 +289,7 @@ gst_raw_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   GstRawParse *rp = GST_RAW_PARSE (parent);
   GstFlowReturn ret = GST_FLOW_OK;
   GstRawParseClass *rp_class = GST_RAW_PARSE_GET_CLASS (rp);
-  guint buffersize, available;
+  guint buffersize;
 
   if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT))) {
     GST_DEBUG_OBJECT (rp, "received DISCONT buffer");
@@ -316,23 +300,16 @@ gst_raw_parse_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
   if (!gst_raw_parse_set_src_caps (rp))
     goto no_caps;
 
-  if (rp->start_segment) {
-    GST_DEBUG_OBJECT (rp, "sending start segment");
-    gst_pad_push_event (rp->srcpad, rp->start_segment);
-    rp->start_segment = NULL;
-  }
-
   gst_adapter_push (rp->adapter, buffer);
 
-  available = gst_adapter_available (rp->adapter);
   if (rp_class->multiple_frames_per_buffer) {
-    buffersize = available;
+    buffersize = gst_adapter_available (rp->adapter);
     buffersize -= buffersize % rp->framesize;
   } else {
     buffersize = rp->framesize;
   }
 
-  while (buffersize > 0 && gst_adapter_available (rp->adapter) >= buffersize) {
+  while (gst_adapter_available (rp->adapter) >= buffersize) {
     buffer = gst_adapter_take_buffer (rp->adapter, buffersize);
 
     ret = gst_raw_parse_push_buffer (rp, buffer);
@@ -361,22 +338,6 @@ gst_raw_parse_loop (GstElement * element)
   GstBuffer *buffer;
   gint size;
 
-  if (G_UNLIKELY (rp->push_stream_start)) {
-    gchar *stream_id;
-    GstEvent *event;
-
-    stream_id =
-        gst_pad_create_stream_id (rp->srcpad, GST_ELEMENT_CAST (rp), NULL);
-
-    event = gst_event_new_stream_start (stream_id);
-    gst_event_set_group_id (event, gst_util_group_id_next ());
-
-    GST_DEBUG_OBJECT (rp, "Pushing STREAM_START");
-    gst_pad_push_event (rp->srcpad, event);
-    rp->push_stream_start = FALSE;
-    g_free (stream_id);
-  }
-
   if (!gst_raw_parse_set_src_caps (rp))
     goto no_caps;
 
@@ -543,8 +504,6 @@ gst_raw_parse_sink_activatemode (GstPad * sinkpad, GstObject * parent,
         }
         rp->segment.duration = duration;
 
-        rp->push_stream_start = TRUE;
-
         result = gst_raw_parse_handle_seek_pull (rp, NULL);
         rp->mode = mode;
       } else {
@@ -726,21 +685,16 @@ gst_raw_parse_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
           break;
         }
 
-        segment.format = GST_FORMAT_TIME;
-
         event = gst_event_new_segment (&segment);
       }
 
       gst_segment_copy_into (&segment, &rp->segment);
 
-      if (rp->start_segment)
-        gst_event_unref (rp->start_segment);
-      rp->start_segment = event;
-      ret = TRUE;
+      ret = gst_pad_push_event (rp->srcpad, event);
       break;
     }
     default:
-      ret = gst_pad_event_default (pad, parent, event);
+      ret = gst_pad_event_default (rp->sinkpad, parent, event);
       break;
   }
 
@@ -828,13 +782,14 @@ gst_raw_parse_handle_seek_pull (GstRawParse * rp, GstEvent * event)
 
     GST_DEBUG_OBJECT (rp, "converted start - stop to time");
 
+    format = GST_FORMAT_TIME;
+
     gst_event_unref (event);
   } else {
+    format = GST_FORMAT_TIME;
     flags = 0;
   }
 
-  format = GST_FORMAT_TIME;
-
   flush = ((flags & GST_SEEK_FLAG_FLUSH) != 0);
 
   /* start flushing up and downstream so that the loop function pauses and we
@@ -941,7 +896,7 @@ gst_raw_parse_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
         ret = gst_raw_parse_handle_seek_pull (rp, event);
       break;
     default:
-      ret = gst_pad_event_default (pad, parent, event);
+      ret = gst_pad_event_default (rp->srcpad, parent, event);
       break;
   }
 
@@ -1024,11 +979,6 @@ gst_raw_parse_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       GstFormat fmt;
 
       ret = TRUE;
-
-      /* try upstream first */
-      if (gst_pad_peer_query (rp->sinkpad, query))
-        break;
-
       gst_query_parse_seeking (query, &fmt, NULL, NULL, NULL);
       if (fmt != GST_FORMAT_TIME && fmt != GST_FORMAT_DEFAULT
           && fmt != GST_FORMAT_BYTES) {
@@ -1050,7 +1000,7 @@ gst_raw_parse_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
     }
     default:
       /* else forward upstream */
-      ret = gst_pad_query_default (pad, parent, query);
+      ret = gst_pad_query_default (rp->sinkpad, parent, query);
       break;
   }
 
