@@ -22,12 +22,10 @@
 #define __GST_AMC_VIDEO_DEC_H__
 
 #include <gst/gst.h>
-#include <gst/gl/gl.h>
 
 #include <gst/video/gstvideodecoder.h>
 
 #include "gstamc.h"
-#include "gstamcsurface.h"
 
 G_BEGIN_DECLS
 
@@ -46,14 +44,6 @@ G_BEGIN_DECLS
 
 typedef struct _GstAmcVideoDec GstAmcVideoDec;
 typedef struct _GstAmcVideoDecClass GstAmcVideoDecClass;
-typedef enum _GstAmcCodecConfig GstAmcCodecConfig;
-
-enum _GstAmcCodecConfig
-{
-  AMC_CODEC_CONFIG_NONE,
-  AMC_CODEC_CONFIG_WITH_SURFACE,
-  AMC_CODEC_CONFIG_WITHOUT_SURFACE,
-};
 
 struct _GstAmcVideoDec
 {
@@ -61,21 +51,20 @@ struct _GstAmcVideoDec
 
   /* < private > */
   GstAmcCodec *codec;
-  GstAmcCodecConfig codec_config;
+  GstAmcBuffer *input_buffers, *output_buffers;
+  gsize n_input_buffers, n_output_buffers;
 
   GstVideoCodecState *input_state;
   gboolean input_state_changed;
 
   /* Output format of the codec */
   GstVideoFormat format;
-  GstAmcColorFormatInfo color_format_info;
+  gint color_format;
+  gint width, height, stride, slice_height;
+  gint crop_left, crop_right;
+  gint crop_top, crop_bottom;
 
-  /* Output dimensions */
-  guint width;
-  guint height;
-
-  guint8 *codec_data;
-  gsize codec_data_size;
+  GstBuffer *codec_data;
   /* TRUE if the component is configured and saw
    * the first buffer */
   gboolean started;
@@ -84,35 +73,15 @@ struct _GstAmcVideoDec
   GstClockTime last_upstream_ts;
 
   /* Draining state */
-  GMutex drain_lock;
-  GCond drain_cond;
+  GMutex *drain_lock;
+  GCond *drain_cond;
   /* TRUE if EOS buffers shouldn't be forwarded */
   gboolean draining;
-  /* TRUE if the component is drained currently */
-  gboolean drained;
 
-  GstAmcSurface *surface;
+  /* TRUE if upstream is EOS */
+  gboolean eos;
 
-  GstGLDisplay *gl_display;
-  GstGLContext *gl_context;
-  GstGLContext *other_gl_context;
-
-  gboolean downstream_supports_gl;
   GstFlowReturn downstream_flow_ret;
-
-  jobject listener;
-  jmethodID set_context_id;
-
-  gboolean gl_mem_attached;
-  GstGLMemory *oes_mem;
-  GError *gl_error;
-  GMutex gl_lock;
-  GCond gl_cond;
-  guint gl_last_rendered_frame;
-  guint gl_pushed_frame_count; /* n buffers pushed */
-  guint gl_ready_frame_count;  /* n buffers ready for GL access */
-  guint gl_released_frame_count;  /* n buffers released */
-  GQueue *gl_queue;
 };
 
 struct _GstAmcVideoDecClass
