@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifndef _GST_DECKLINK_SINK_H_
@@ -29,7 +29,6 @@ G_BEGIN_DECLS
 #define GST_TYPE_DECKLINK_SINK   (gst_decklink_sink_get_type())
 #define GST_DECKLINK_SINK(obj)   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_DECKLINK_SINK,GstDecklinkSink))
 #define GST_DECKLINK_SINK_CLASS(klass)   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DECKLINK_SINK,GstDecklinkSinkClass))
-#define GST_DECKLINK_SINK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_DECKLINK_SINK, GstDecklinkSinkClass))
 #define GST_IS_DECKLINK_SINK(obj)   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_DECKLINK_SINK))
 #define GST_IS_DECKLINK_SINK_CLASS(obj)   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_DECKLINK_SINK))
 
@@ -58,10 +57,10 @@ struct _GstDecklinkSink
   GstPad *videosinkpad;
   GstPad *audiosinkpad;
 
-  GMutex mutex;
-  GCond  cond;
-  GMutex audio_mutex;
-  GCond  audio_cond;
+  GMutex *mutex;
+  GCond *cond;
+  GMutex *audio_mutex;
+  GCond *audio_cond;
   int queued_frames;
   gboolean stop;
   gboolean video_eos;
@@ -81,15 +80,15 @@ struct _GstDecklinkSink
 
   /* properties */
   GstDecklinkModeEnum mode;
-  int device_number;
+  int device;
 
 #ifdef _MSC_VER
   gboolean comInitialized;
-  GMutex   com_init_lock;
-  GMutex   com_deinit_lock;
-  GCond    com_initialized;
-  GCond    com_uninitialize;
-  GCond    com_uninitialized;
+  GMutex   *com_init_lock;
+  GMutex   *com_deinit_lock;
+  GCond    *com_initialized;
+  GCond    *com_uninitialize;
+  GCond    *com_uninitialized;
 #endif /* _MSC_VER */
 };
 
